'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _AbstractClientStore = require('express-brute/lib/AbstractClientStore');

var _AbstractClientStore2 = _interopRequireDefault(_AbstractClientStore);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _sequelize = require('sequelize');

var _sequelize2 = _interopRequireDefault(_sequelize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } // Node modules


class Store {
  constructor(sequelize, table, options, ...args) {
    _AbstractClientStore2.default.apply(this, args);

    this.defaults = {
      prefix: '',
      logging: false
    };

    this.options = Object.assign({}, this.defaults, options);

    this.table = sequelize.define(table, {
      _id: {
        type: _sequelize2.default.STRING,
        unique: true
      },
      expires: {
        type: _sequelize2.default.DATE
      },
      firstRequest: {
        type: _sequelize2.default.DATE
      },
      lastRequest: {
        type: _sequelize2.default.DATE
      },
      count: {
        type: _sequelize2.default.INTEGER
      }
    });
  }

  migrate() {
    var _this = this;

    return _asyncToGenerator(function* () {
      try {
        yield _this.table.sync();
      } catch (error) {
        // eslint-disable-next-line no-console
        console.error(`Failed to initialize Store - table ${_this.table}`);
      }
    })();
  }

  set(key, value, lifetime) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const id = _this2.options.prefix + key;

      const expiration = lifetime ? (0, _moment2.default)().add(lifetime, 'seconds').toDate() : null;

      const doc = yield _this2.table.find({
        where: {
          _id: id
        }
      });

      if (doc) {
        // eslint-disable-next-line no-underscore-dangle
        doc._id = id;
        doc.count = value.count;
        doc.lastRequest = value.lastRequest;
        doc.firstRequest = value.firstRequest;
        doc.expires = expiration;

        return doc.save();
      }

      return _this2.table.create({
        _id: id,
        count: value.count,
        lastRequest: value.lastRequest,
        firstRequest: value.firstRequest,
        expires: expiration
      });
    })();
  }

  get(key) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const id = _this3.options.prefix + key;

      const doc = yield _this3.table.find({
        where: {
          _id: id
        }
      });

      if (doc && new Date(doc.expires).getTime() < new Date().getTime()) {
        yield _this3.table.destroy({
          where: {
            _id: id
          }
        });
        return null;
      }

      if (doc) {
        return {
          count: doc.count,
          lastRequest: new Date(doc.lastRequest),
          firstRequest: new Date(doc.firstRequest)
        };
      }

      return null;
    })();
  }

  reset(key) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      const id = _this4.options.prefix + key;

      const doc = yield _this4.table.destroy({
        where: {
          _id: id
        }
      });

      return doc;
    })();
  }
}

exports.default = Store;