// Node modules
import chai, { assert } from 'chai';
import chaiAsPromised from 'chai-as-promised';
import Sequelize from 'sequelize';

// Module to test
import Store from '../src';

chai.use(chaiAsPromised);

const sequelize = new Sequelize('database', 'username', 'password', {
  dialect: 'sqlite',
  storage: ':memory:',
  logging: false,
});

describe('store', () => {
  let store;

  beforeEach(async () => {
    store = new Store(sequelize, 'brutes', {});
    await store.migrate();
  });

  describe('set', () => {
    it('should be able to set a value', async () => {
      await store.set('foo', {
        count: 123,
      }, 1000);

      const result = await store.table.find({
        where: {
          _id: 'foo',
        },
      });

      assert.equal(result.count, 123);
    });
  });

  describe('get', () => {
    it('should be able to get a value', async () => {
      await store.set('foo', {
        count: 123,
      }, 1000);

      const result = await store.get('foo');

      assert.equal(result.count, 123);
    });

    it('should return undefined if expired', async () => {
      await store.set('foo', {
        count: 123,
      }, 0);

      const result = await store.get('foo');

      assert.equal(result, null);
    });

    it('should delete the doc if expired', async () => {
      await store.set('foo', {
        count: 123,
      }, 0);

      await store.get('foo');

      const result = await store.table.find({
        where: {
          _id: 'foo',
        },
      });

      assert.equal(result, null);
    });
  });

  describe('reset', () => {
    it('should be able to reset', async () => {
      await store.set('foo', {
        count: 123,
      }, 1000);

      await store.reset('foo');

      const result = await store.table.find({
        where: {
          _id: 'foo',
        },
      });

      assert.equal(result, null);
    });
  });
});
